@@ -155,6 +155,11 @@ void load_labwc_config (void)
                     if (strstr (line, "no")) mons[mon].enabled = FALSE;
                     else mons[mon].enabled = TRUE;
                 }
+                else if (strstr (line, "Scale"))
+                {
+                    sscanf (line, "  Scale: %f", &f);
+                    mons[mon].scale = f;
+                }
             }
             else if (line[4] != ' ')
             {
@@ -246,14 +251,14 @@ static int write_config (FILE *fp)
         }
         else if (mons[m].freq == 0.0)
         {
-            fprintf (fp, "\t\toutput %s enable mode --custom %dx%d position %d,%d transform %s\n",
-                mons[m].name, mons[m].width, mons[m].height,
+            fprintf (fp, "\t\toutput %s enable scale %f mode --custom %dx%d position %d,%d transform %s\n",
+                mons[m].name, mons[m].scale, mons[m].width, mons[m].height,
                 mons[m].x, mons[m].y, orients[mons[m].rotation / 90]);
         }
         else
         {
-            fprintf (fp, "\t\toutput %s enable mode %dx%d@%.3f position %d,%d transform %s\n",
-                mons[m].name, mons[m].width, mons[m].height, mons[m].freq,
+            fprintf (fp, "\t\toutput %s enable scale %f mode %dx%d@%.3f position %d,%d transform %s\n",
+                mons[m].name, mons[m].scale, mons[m].width, mons[m].height, mons[m].freq,
                 mons[m].x, mons[m].y, orients[mons[m].rotation / 90]);
         }
     }
@@ -411,7 +416,7 @@ void load_labwc_touchscreens (void)
 {
     char *infile;
 
-    // read_touchscreen_xml ("/etc/xdg/labwc/rc.xml");
+    read_touchscreen_xml ("/etc/xdg/labwc/rc.xml");
 
     infile = g_build_filename (g_get_user_config_dir (), "labwc/rc.xml", NULL);
     read_touchscreen_xml (infile);
@@ -503,14 +508,11 @@ void save_labwc_touchscreens (void)
     g_free (infile);
     g_free (outfile);
 
-    infile = g_build_filename (g_get_user_config_dir (), "labwc/rc.xml", NULL);
     outfile = g_build_filename (g_get_user_config_dir (), "labwc/rcgreeter.xml", NULL);
-    // cmd = g_strdup_printf ("cp /etc/xdg/labwc-greeter/rc.xml %s", outfile);
-    cmd = g_strdup_printf ("cp %s %s", infile, outfile);
+    cmd = g_strdup_printf ("cp /etc/xdg/labwc-greeter/rc.xml %s", outfile);
     system (cmd);
     g_free (cmd);
     write_touchscreens (outfile);
-    g_free (infile);
     g_free (outfile);
 }
 
